@@ -6,12 +6,15 @@ This provider implements all funcionalities from asyncpg (cursors, transactions,
 
 import asyncio
 import json
-import logging
 import time
 from datetime import datetime
 import traceback
 
 import asyncpg
+from asyncpg.pgproto import pgproto
+from dateutil.relativedelta import relativedelta
+from typing import Tuple
+
 from asyncpg.exceptions import (
     ConnectionDoesNotExistError,
     FatalPostgresError,
@@ -54,27 +57,31 @@ from asyncdb.providers.sql import (
     baseCursor
 )
 
-logger = logging.getLogger(__name__)
-
 max_cached_statement_lifetime = 600
 max_cacheable_statement_size = 1024 * 15
 
 
 class pgPool(BasePool):
-    _max_queries = 300
+    _max_queries = 100
     _dsn = "postgres://{user}:{password}@{host}:{port}/{database}"
     _server_settings = {}
     init_func = None
     setup_func = None
-    _max_clients = 500
+    _max_clients = 1500
+    _min_size = 5
+    application_name = 'Navigator'
 
     def __init__(self, dsn="", loop=None, params={}, **kwargs):
         super(pgPool,
               self).__init__(dsn=dsn, loop=loop, params=params, **kwargs)
         if "server_settings" in kwargs:
             self._server_settings = kwargs["server_settings"]
+        if 'application_name' in self._server_settings:
+            self.application_name = self._server_settings['application_name']
         if "max_clients" in kwargs:
             self._max_clients = kwargs["max_clients"]
+        if "min_size" in kwargs:
+            self._min_size = kwargs["min_size"]
 
     def get_event_loop(self):
         return self._loop
@@ -96,14 +103,54 @@ class pgPool(BasePool):
             return json.loads(value)
 
         await connection.set_type_codec(
-            "json", encoder=_encoder, decoder=_decoder, schema="pg_catalog"
+            "json",
+            encoder=_encoder,
+            decoder=_decoder,
+            schema="pg_catalog"
         )
         await connection.set_type_codec(
-            "jsonb", encoder=_encoder, decoder=_decoder, schema="pg_catalog"
+            "jsonb",
+            encoder=_encoder,
+            decoder=_decoder,
+            schema="pg_catalog"
         )
         await connection.set_builtin_type_codec(
             "hstore", codec_name="pg_contrib.hstore"
         )
+
+        def timedelta_decoder(delta: Tuple) -> relativedelta:
+            return relativedelta(months=delta[0], days=delta[1], microseconds=delta[2])
+
+        def timedelta_encoder(delta: relativedelta):
+            ndelta = delta.normalized()
+            return (
+                ndelta.years * 12 + ndelta.months, ndelta.days,
+                (ndelta.hours * 3600 + ndelta.minutes * 60 + ndelta.seconds)
+                * 1_000_000 + ndelta.microseconds
+            )
+
+        await connection.set_type_codec(
+            'interval',
+            schema='pg_catalog',
+            encoder=timedelta_encoder,
+            decoder=timedelta_decoder,
+            format='tuple'
+        )
+
+        def _uuid_encoder(value):
+            if value:
+                val = uuid.UUID(value).bytes
+            else:
+                val = b''
+            return val
+
+        await connection.set_type_codec(
+            "uuid",
+            encoder=_uuid_encoder,
+            decoder=lambda u: pgproto.UUID(u),
+            schema='pg_catalog',
+            format='binary'
+        )
         if self.init_func:
             try:
                 await self.init_func(connection)
@@ -117,11 +164,11 @@ class pgPool(BasePool):
 
     # Create a database connection pool
     async def connect(self):
-        logger.debug("AsyncPg (Pool): Connecting to {}".format(self._dsn))
+        self._logger.debug("AsyncPg (Pool): Connecting to {}".format(self._dsn))
         try:
             # TODO: pass a setup class for set_builtin_type_codec and a setup for add listener
             server_settings = {
-                "application_name": "Navigator",
+                "application_name": self.application_name,
                 "idle_in_transaction_session_timeout": "600",
                 "tcp_keepalives_idle": "600",
                 "max_parallel_workers": "16",
@@ -130,10 +177,10 @@ class pgPool(BasePool):
             self._pool = await asyncpg.create_pool(
                 dsn=self._dsn,
                 max_queries=self._max_queries,
-                min_size=10,
+                min_size=self._min_size,
                 max_size=self._max_clients,
-                max_inactive_connection_lifetime=10,
-                timeout=self._timeout,
+                max_inactive_connection_lifetime=100,
+                timeout=5,
                 command_timeout=self._timeout,
                 init=self.init_connection,
                 setup=self.setup_connection,
@@ -209,18 +256,18 @@ class pgPool(BasePool):
     Release a connection from the pool
     """
 
-    async def release(self, connection=None, timeout=10):
+    async def release(self, connection=None, timeout=5):
         if not connection:
             conn = self._connection
         else:
             conn = connection
         if isinstance(connection, pg):
             conn = connection.engine()
+        if not conn:
+            return True
         try:
-            release = asyncio.create_task(self._pool.release(conn, timeout=10))
-            #await self._pool.release(conn, timeout = timeout)
-            #release = asyncio.ensure_future(release, loop=self._loop)
-            await asyncio.wait_for(release, timeout=timeout, loop=self._loop)
+            await self._pool.release(conn, timeout=timeout)
+            return True
         except InterfaceError as err:
             raise ProviderError("Release Interface Error: {}".format(str(err)))
         except InternalClientError as err:
@@ -243,6 +290,7 @@ class pgPool(BasePool):
             try:
                 if self._connection:
                     await self._pool.release(self._connection, timeout=2)
+                    self._connection = None
             except (InternalClientError, InterfaceError) as err:
                 raise ProviderError(
                     "Release Interface Error: {}".format(str(err))
@@ -251,19 +299,25 @@ class pgPool(BasePool):
                 raise ProviderError("Release Error: {}".format(str(err)))
             try:
                 if gracefully:
+                    await self._pool.expire_connections()
                     close = asyncio.create_task(self._pool.close())
-                    close.add_done_callback(_handle_done_tasks)
-                    await asyncio.wait_for(close, timeout=timeout, loop=self._loop)
-                else:
-                    await self._pool.close()
-            except asyncio.exceptions.TimeoutError as err:
-                print(traceback.format_exc())
+                    #close.add_done_callback(_handle_done_tasks)
+                    try:
+                        await asyncio.wait_for(
+                            close,
+                            timeout=timeout,
+                            loop=self._loop
+                        )
+                    except asyncio.exceptions.TimeoutError as err:
+                        #print(traceback.format_exc())
+                        pass
+                # # until end, close the pool correctly:
+                self._pool.terminate()
             except Exception as err:
                 error = f'Pool Exception: {err.__class__.__name__}: {err}'
                 print("Pool Error: {}".format(error))
                 raise ProviderError("Pool Error: {}".format(error))
             finally:
-                self._pool.terminate()
                 self._connected = False
 
     """
@@ -274,12 +328,23 @@ class pgPool(BasePool):
         try:
             if self._connection:
                 await self._pool.release(self._connection, timeout=2)
+                self._connection = None
         except InterfaceError as err:
             raise ProviderError("Release Interface Error: {}".format(str(err)))
         except Exception as err:
             raise ProviderError("Release Error: {}".format(str(err)))
         try:
-            await self._pool.close()
+            await self._pool.expire_connections()
+            close = asyncio.create_task(self._pool.close())
+            #close.add_done_callback(_handle_done_tasks)
+            try:
+                await asyncio.wait_for(
+                    close,
+                    timeout=timeout,
+                    loop=self._loop
+                )
+            except asyncio.exceptions.TimeoutError as err:
+                pass
         except Exception as err:
             print("Pool Closing Error: {}".format(str(err)))
         finally:
@@ -288,7 +353,7 @@ class pgPool(BasePool):
 
     def terminate(self, gracefully=True):
         self._loop.run_until_complete(
-            asyncio.wait_for(self.close(), timeout=5)
+            self.wait_close(gracefully=gracefully, timeout=10)
         )
 
     """
@@ -330,11 +395,14 @@ class pg(SQLProvider):
     _initialized_on = None
     _query_raw = "SELECT {fields} FROM {table} {where_cond}"
     _server_settings = {}
+    application_name: str = 'Navigator'
 
     def __init__(self, dsn: str = '', loop = None, params={}, **kwargs):
-        super(pg, self).__init__(dsn, loop, params, **kwargs)
+        super(pg, self).__init__(dsn=dsn, loop=loop, params=params, **kwargs)
         if "server_settings" in kwargs:
             self._server_settings = kwargs["server_settings"]
+        if 'application_name' in self._server_settings:
+            self.application_name = self._server_settings['application_name']
 
     async def close(self, timeout=5):
         """
@@ -343,7 +411,7 @@ class pg(SQLProvider):
         try:
             if self._connection:
                 if not self._connection.is_closed():
-                    logger.debug(
+                    self._logger.debug(
                         "Closing Connection, id: {}".format(
                             self._connection.get_server_pid()
                         )
@@ -390,7 +458,7 @@ class pg(SQLProvider):
             return json.loads(value)
 
         server_settings = {
-            "application_name": "AsyncDB",
+            "application_name": self.application_name,
             "idle_in_transaction_session_timeout": "600",
             "tcp_keepalives_idle": "600",
             "max_parallel_workers": "16",
@@ -498,8 +566,9 @@ class pg(SQLProvider):
             try:
                 stmt = await asyncio.shield(self._connection.prepare(sentence))
                 try:
-                    # print(stmt.get_attributes())
-                    self._columns = [a.name for a in stmt.get_attributes()]
+                    self._attributes = stmt.get_attributes()
+                    self._columns = [a.name for a in self._attributes]
+                    #self._columns = [a.name for a in stmt.get_attributes()]
                     self._prepared = stmt
                     self._parameters = stmt.get_parameters()
                 except TypeError:
@@ -564,7 +633,8 @@ class pg(SQLProvider):
             await self.connection()
         try:
             stmt = await self._connection.prepare(sentence)
-            self._columns = [a.name for a in stmt.get_attributes()]
+            self._attributes = stmt.get_attributes()
+            self._columns = [a.name for a in self._attributes]
             self._result = await stmt.fetchrow()
         except RuntimeError as err:
             error = "Runtime on Query Row Error: {}".format(str(err))
@@ -606,11 +676,13 @@ class pg(SQLProvider):
         except InterfaceWarning as err:
             error = "Interface Warning: {}".format(str(err))
             raise ProviderError(error)
-            return [None, error]
+        except asyncpg.exceptions.DuplicateTableError as err:
+            error = err
+            raise ProviderError(error)
         except Exception as err:
             error = "Error on Execute: {}".format(str(err))
-            self._loop.call_exception_handler(err)
-            raise [None, error]
+            #self._loop.call_exception_handler(err)
+            raise ProviderError(error)
         finally:
             return [result, error]
 
@@ -621,15 +693,14 @@ class pg(SQLProvider):
         if not self._connection:
             await self.connection()
         try:
-            async with self._connection.transaction():
-                await self._connection.executemany(sentence, *args)
+            await self._connection.executemany(sentence, *args)
         except InterfaceWarning as err:
             error = "Interface Warning: {}".format(str(err))
             raise ProviderError(error)
             return False
         except Exception as err:
             error = "Error on Execute: {}".format(str(err))
-            self._loop.call_exception_handler(err)
+            #self._loop.call_exception_handler(err)
             raise Exception(error)
         finally:
             return error
@@ -764,6 +835,9 @@ class pg(SQLProvider):
         """
         if not self._connection:
             await self.connection()
+        if self._transaction:
+            # a transaction exists:
+            await self._transaction.commit()
         try:
             result = await self._connection.copy_to_table(
                 table_name=table,
@@ -800,6 +874,9 @@ class pg(SQLProvider):
         """
         if not self._connection:
             await self.connection()
+        if self._transaction:
+            # a transaction exists:
+            await self._transaction.commit()
         try:
             result = await self._connection.copy_records_to_table(
                 table_name=table,
