@@ -89,7 +89,7 @@ class rethink(BaseProvider):
 
 
     async def connection(self):
-        logging.debug(
+        self._logger.debug(
             "RT Connection to host {} on port {} to database {}".format(
                 self._params["host"], self._params["port"], self._params["db"]
             )
@@ -121,10 +121,6 @@ class rethink(BaseProvider):
                 self._connected = True
         return self
 
-    @property
-    def is_connected(self):
-        return self._connected
-
     def engine(self):
         return self._engine
 
@@ -139,7 +135,7 @@ class rethink(BaseProvider):
             self._connection = None
 
     def terminate(self):
-        logging.debug("Closing Rethink Connection")
+        self._logger.debug("Closing Rethink Connection")
         self._loop.run_until_complete(self.close())
 
     async def release(self):
@@ -334,7 +330,16 @@ class rethink(BaseProvider):
     """
 
     async def test_connection(self):
-        return await self._engine.db_list().run(self._connection)
+        result = None
+        error = None
+        try:
+            result = await self._engine.db_list().run(self._connection)
+            if result:
+                return [result, error]
+        except Exception as err:
+            return [None, err]
+        finally:
+            return [result, error]
 
     def execute(self):
         pass
